void function() {
	var rate = 1.5;
	var configs = {
		isLocalTesting : location.hostname.indexOf("192.168.1.") === 0,
		map : {
			width : 1500*rate,
			height : 1500*rate,
			cellsCount : 225,
			styles : {
				containerClassName : "container",
				mapWrapperClassName : "map-wrapper",
				overlayClassName : "overlay",
				overlayTransitionDuration : "500ms",
				touchLayerClassName : "touch-layer",
				spritesUrl : "images/map-sprites2/",
				spritesWidth : 100*rate,
				spritesHeight : 100*rate,
				selectedWrapperClassName : "selected-station-wrapper",
				selectedClassName : "selected-station",
				pinsWrapperClassName : "pins-wrapper",
				pinDisabledClassName : "pin-disabled",
				pinFromClassName : "pin-from",
				pinToClassName : "pin-to",
				pinTransferClassName : "pin-transfer",
				pinsUrl : "images/pins/pin_s.png",
				pinsDisabledUrl : "images/pins/pin_disabled_s.png",
				pinsFromUrl : "images/pins/pin_from_s.png",
				pinsToUrl : "images/pins/pin_to_s.png",
				pinsTransferUrl : "images/pins/pin_transform_s.png",
				pinWidth : 90*rate,
				pinHeight : 90*rate
			}
		},
		language : "cn",
		deviceWidth : window.screen.width,
		deviceHeight : window.screen.height,
		marginTop : 0,
		marginBottom : 0,
		popOverViewWidth : 200,
		maxGap : 80,
		initialScale : .4
	};

	var touch;
	var pinsCollection;
	var mapView;
	var animationId = null;

	function init() {
		location.hash = "";

		touch = Touch().getInstance();

		function start() {
			setTimeout(function() {
				broadcast("onWebViewReady");
			}, 10);
		}

		if(document.readyState === "complete") start();
		else document.addEventListener("DOMContentLoaded", start, false);
		ctrl();
	}

	function Touch() {
		function init() {
			var eventsArr = [];

			var isTouchDevice = "ontouchstart" in document;
			var fire = function() {
				if(isTouchDevice) {
					return {
						start : "touchstart",
						move : "touchmove",
						end : "touchend"
					};
				} else {
					return {
						start : "mousedown",
						move : "mousemove",
						end : "mouseup"
					};
				}
			}();

			return {
				isTouchDevice : function() {
					return isTouchDevice;
				},

				add : function(trigger, onStart, onMove, onEnd, useCapture) {
					var eventId = (new Date()).getTime();

					if(typeof useCapture === "undefined") useCapture = false;
					if(typeof trigger === "undefined") trigger = document;

					function onStartHandler(ev) {
						if(typeof onStart === "function") onStart(ev);
						trigger.addEventListener(fire.move, onMoveHandler, useCapture);
						trigger.addEventListener(fire.end, onEndHandler, useCapture);
					}

					function onMoveHandler(ev) {
						if(typeof onMove === "function") onMove(ev);
						trigger.removeEventListener(fire.end, onEndHandler);
					}

					function onEndHandler(ev) {
						if(typeof onEnd === "function") onEnd(ev);
						trigger.removeEventListener(fire.move, onMoveHandler);
						trigger.removeEventListener(fire.end, onEndHandler);
					}

					trigger.addEventListener(fire.start, onStartHandler, useCapture);

					eventsArr.push({
						eventId : eventId,
						trigger : trigger,
						onStart : onStartHandler,
						onMove : onMoveHandler,
						onEnd : onEndHandler
					});

					return eventId;
				},

				remove : function(eventId) {
					if(typeof eventId === "undefined") throw "undefined eventId";

					var index;
					var targetEvent;

					for(index in eventsArr) {
						var thisEvent = eventsArr[index];
						if(thisEvent.eventId === eventId) {
							targetEvent = thisEvent;
							break;
						}
					}

					if(typeof index === "undefined") throw "Can not find eventId: " + eventId + "!";

					targetEvent.trigger.removeEventListener(fire.start, targetEvent.onStart);
					targetEvent.trigger.removeEventListener(fire.move, targetEvent.onMove);
					targetEvent.trigger.removeEventListener(fire.end, targetEvent.onEnd);

					eventsArr.splice(index, 1);
				}
			}
		}

		return {
			getInstance : init
		};
	}

	function MapView() {
		var container = function() {
			var elem = document.createElement("div");
			elem.className = configs.map.styles.containerClassName;
			elem.style.cssText = "width:" + configs.map.width + "px; height:" + configs.map.height + "px; padding:" + configs.marginTop + "px " + configs.popOverViewWidth +  "px " + configs.marginBottom + "px;";
			// elem.style.visibility = "hidden";
			document.body.appendChild(elem);
			return elem;
		}();

		var mapWrapper = function() {
			var elem = document.createElement("div");
			elem.className = configs.map.styles.mapWrapperClassName;
			container.appendChild(elem);
			return elem;
		}();
		var touchLayer = function() {
			touchLayer = document.createElement("div");
			touchLayer.className = configs.map.styles.touchLayerClassName;
			container.appendChild(touchLayer);
			return touchLayer;
		}();
		var overlay = null;
		var pinsWrapper = null;
		var selectedWrapper = null;

		var touchDocumentEventId;

		/*
		function transition(targetElem, callback, timeout) {
			if(typeof callback !== "function") return;

			var timeoutId;
			var transitionEnd = "transitionEnd" in document.documentElement.style ? "transitionEnd" : "webkitTransitionEnd";

			function stop() {
				if(timeoutId) clearTimeout(timeoutId);
				targetElem.removeEventListener(transitionEnd, callback);
				callback();
			}

			timeoutId = setTimeout(function() {
				timeoutId = null;
				stop();
			}, timeout);

			targetElem.addEventListener(transitionEnd, stop, false);
		}
		*/

		function delay(callback) {
			setTimeout(callback, 5);
		}

		return {
			initCss : function() {
				var styleSheet = document.styleSheets[0];

				var halfPinWidth = configs.map.styles.pinWidth / 2;
				var halfPinHeight = configs.map.styles.pinHeight / 2;

				function getWebkitStyleName(styleName) {
					return styleName in document.documentElement.style ? styleName : "-webkit-" + styleName;
				}

				// .container
				styleSheet.addRule("." + configs.map.styles.containerClassName, 'width: ' + configs.map.width + 'px; height: ' + configs.map.height + 'px; position: relative; ' + getWebkitStyleName("user-select") + ': none;');
				// .map-wrapper
				styleSheet.addRule("." + configs.map.styles.mapWrapperClassName, 'width: ' + configs.map.width + 'px; height: ' + configs.map.height + 'px;');
				// .map-wrapper img
				styleSheet.addRule("." + configs.map.styles.mapWrapperClassName + " img", 'width: ' + configs.map.styles.spritesWidth + 'px; height: ' + configs.map.styles.spritesHeight + 'px; display: block; float: left; ' + getWebkitStyleName("user-select") + ': none;');
				// .pins-wrapper
				styleSheet.addRule("." + configs.map.styles.pinsWrapperClassName, 'position: absolute; top: 0; right: 0; bottom: 0; left: 0; z-index: 101;');
				// .pins-wrapper div
				styleSheet.addRule("." + configs.map.styles.pinsWrapperClassName + " div", 'width: ' + configs.map.styles.pinWidth + 'px; height: ' + configs.map.styles.pinHeight + 'px; display: block; position: absolute; margin: -' + halfPinHeight + 'px 0 0 -' + halfPinWidth + 'px;  background: url(' + configs.map.styles.pinsUrl + ') center center no-repeat; ' + getWebkitStyleName("background-size") + ': ' + configs.map.styles.pinWidth + 'px ' + configs.map.styles.pinHeight + 'px;');
				// .pins-wrapper .disabled
				styleSheet.addRule("." + configs.map.styles.pinsWrapperClassName + " ." + configs.map.styles.pinDisabledClassName, 'background-image: url(' + configs.map.styles.pinsDisabledUrl + ');');
				// .pins-wrapper .from
				styleSheet.addRule("." + configs.map.styles.pinsWrapperClassName + " ." + configs.map.styles.pinFromClassName, 'background-image: url(' + configs.map.styles.pinsFromUrl + ');');
				// .pins-wrapper .to
				styleSheet.addRule("." + configs.map.styles.pinsWrapperClassName + " ." + configs.map.styles.pinToClassName, 'background-image: url(' + configs.map.styles.pinsToUrl + ');');
				// .pins-wrapper .transfer
				styleSheet.addRule("." + configs.map.styles.pinsWrapperClassName + " ." + configs.map.styles.pinTransferClassName, 'background-image: url(' + configs.map.styles.pinsTransferUrl + ');');
				// .overlay
				styleSheet.addRule("." + configs.map.styles.overlayClassName, 'position: absolute; top: 0; right: 0; bottom: 0; left: 0; z-index: 100; background-color: white; opacity: 0; ' + getWebkitStyleName("transition") + ': opacity ' + configs.map.styles.overlayTransitionDuration + ';');
				// .touchLayer
				styleSheet.addRule("." + configs.map.styles.touchLayerClassName, 'position: absolute; top: 0; right: 0; bottom: 0; left: 0; z-index: 200; background-color: transparent;');
			},

			addCells : function() {
				var documentFragment = document.createDocumentFragment("div");
				for(var i = 1, l = configs.map.cellsCount + 1; i < l; i++) {
					var n = i < 10 ? "0" + i : i;
					var cellImg = document.createElement("img");
					cellImg.src = configs.map.styles.spritesUrl + "map-" + n + ".png";
					documentFragment.appendChild(cellImg);
				}
				mapWrapper.appendChild(documentFragment);
			},

			scrollToCenter : function() {
				var x = (configs.map.width + (2 * configs.popOverViewWidth) - (configs.deviceWidth / configs.initialScale)) / 2;
				var y = (configs.map.height - ((2 * configs.marginTop) / configs.initialScale) + configs.marginBottom - (configs.deviceHeight / configs.initialScale)) / 2;

				if(x < 0) x = 0;
				if(y < 0) y = 0;

				window.scroll(x, y);
			},

			hideWebview : function() {
				container.style.visibility = "hidden";
			},

			showWebview : function() {
				container.style.visibility = "visible";
			},

			hasOverlay : function() {
				return overlay ? true : false;
			},

			hasPinsWrapper : function() {
				return pinsWrapper ? true : false;
			},

			createOverlay : function() {
				overlay = document.createElement("div");
				overlay.className = configs.map.styles.overlayClassName;
				container.appendChild(overlay);
				return overlay;
			},

			createPinsWrapper : function() {
				pinsWrapper = document.createElement("div");
				pinsWrapper.className = configs.map.styles.pinsWrapperClassName;
				container.appendChild(pinsWrapper);
				return pinsWrapper;
			},

			removePinsWrapper : function() {
				container.removeChild(pinsWrapper);
				pinsWrapper = null;
			},

			createSelectedWrapper : function () {
				selectedWrapper = document.createElement("div");
				selectedWrapper.className = configs.map.styles.selectedWrapperClassName;
				container.appendChild(selectedWrapper);
				return selectedWrapper;
			},

			showOverlay : function(callback) {
				overlay.style.display = "block";
				delay(function() {
					overlay.style.opacity = ".5";
					if(typeof callback === "function") callback();
				});
			},

			hideOverlay : function() {
				overlay.style.display = "none";
				overlay.style.opacity = "0";
			},

			addPin : function(data, flag) {
				pinsCollection.scalePin(pinsCollection.createPin(data, pinsWrapper, flag));
			},

			pick : function (data, flag) {
				pinsCollection.createPin(data, pinsWrapper, flag.flag);
			},

			select : function (data) {
				function addEl(className) {
					var x = parseInt(data.x) + parseInt(configs.popOverViewWidth);
					var y = parseInt(data.y) + parseInt(configs.marginTop);
					var el = document.createElement("span");
					if (className) {
						el.className = className;
						el.style.cssText = "left:" + x + "px; " + "top:" + y + "px";
					}
					
					return el;
				}

				var wrapper = addEl(configs.map.styles.selectedClassName);
				wrapper.appendChild(addEl());
				wrapper.appendChild(addEl());

				selectedWrapper.appendChild(wrapper);
			},

			deselect : function () {
				var el = selectedWrapper.querySelector("." + configs.map.styles.selectedClassName);
				el.parentNode.removeChild(el);
			},

			highlight : function(data) {
				var x = parseInt(data.x) + parseInt(configs.popOverViewWidth);
				var y = parseInt(data.y) + parseInt(configs.marginTop);
				var pinElem = document.createElement("div");
				if(data.isOnConstruction === false || data.isOnConstruction === "false") pinElem.className = configs.map.styles.pinDisabledClassName;
				pinElem.style.cssText = "left:" + x + "px; " + "top:" + y + "px";
				pinsWrapper.appendChild(pinElem);
			},

			addTouchMapEvent : function(onStart, onMove, onEnd) {
				touch.add(touchLayer, onStart, onMove, onEnd);
			},

			addTouchDocumentEvent : function(onStart, onMove, onEnd) {
				touchDocumentEventId = touch.add(document, onStart, onMove, onEnd);
			},

			removeTouchDocumentEvent : function() {
				touch.remove(touchDocumentEventId);
			}
		}
	}

	function PinsCollection() {
		var collection = [];
		var transform = function() {
			return "transform" in document.documentElement.style ? "transform" : "webkitTransform";
		}();

		var scaleRatio = 1;

		return {
			createPin : function(data, pinsWrapper, flag) {
				var x = parseInt(data.x) + parseInt(configs.popOverViewWidth);
				var y = parseInt(data.y) + parseInt(configs.marginTop);
				var pinElem = document.createElement("div");
				if(data.isOnConstruction === false || data.isOnConstruction === "false") pinElem.className = configs.map.styles.pinDisabledClassName;
				else {
					if (flag === "from") pinElem.className = configs.map.styles.pinFromClassName;
					else if (flag === "to") pinElem.className = configs.map.styles.pinToClassName;
					else if (flag === "transfer") pinElem.className = configs.map.styles.pinTransferClassName;
				}
				pinElem.style.cssText = "left:" + x + "px; " + "top:" + y + "px";
				if (!mapView.hasPinsWrapper()) pinsWrapper = mapView.createPinsWrapper();
				pinsWrapper.appendChild(pinElem);
				collection.push(pinElem);
				return pinElem;
			},

			clear : function() {
				collection = [];
			},

			scalePin : function(pinElem) {
				if(collection.length === 0) return;
				pinElem.style[transform] = 'scale(' + scaleRatio + ')';
			},

			scaleAllPins : function(ratio) {
				var length = collection.length;
				scaleRatio = 1 / ratio;
				if(length === 0) return;

				for(var i = 0; i < length; i++) {
					collection[i].style[transform] = 'scale(' + scaleRatio + ')';
				}
			}
		};
	}

	var Station = function (node) {
		for (var key in node) {
			if (node.hasOwnProperty(key)) {
				this[key] = node[key];
			}
		}
	};
	Station.prototype.pushLine = function (lineNumber) {
		this.lines.push(lineNumber);
	};

	function broadcast(eventType, contentObj) {
		if(typeof eventType === "undefined") return;

		var res = {
			eventType : eventType,
			timeStamp : (new Date()).getTime()
		};
		if(typeof contentObj !== "undefined") res.content = contentObj;

		res = JSON.stringify(res);
		if(configs.isLocalTesting) console.log(res);
		else {
		       location.hash = res;
               window.Android.clickImage(JSON.stringify(res));
         }
	}

	function ctrl() {
		var appearance = {};
		var allStationsArr = [];
		var pixelRatio = typeof window.devicePixelRatio === "undefined" ? 1 : window.devicePixelRatio;

		window.WEBVIEW = {};
		WEBVIEW.start = start;

		function start(mapData, settings) {
			initAppearance(settings);
			initMapData(mapData);

			mapView = MapView();
			mapView.initCss();
			mapView.addCells();
			mapView.scrollToCenter();

			mapView.createPinsWrapper();
			mapView.createSelectedWrapper();

			addTouchEvent();
			pinsCollection = PinsCollection();
			addInterface();
			broadcast("onMapReady");
		}

		function initAppearance(settings) {
			if(typeof settings === "string") settings = JSON.parse(settings);

			if(typeof settings.language !== "undefined" && settings.language) appearance.language = settings.language;
			if(typeof settings.width !== "undefined" && settings.width) configs.deviceWidth = Number(settings.width);
			if(typeof settings.height !== "undefined" && settings.height) configs.deviceHeight = Number(settings.height);
			if(typeof settings.marginTop !== "undefined" && settings.marginTop) configs.marginTop = Number(settings.marginTop);
			if(typeof settings.marginBottom !== "undefined" && settings.marginBottom) configs.marginBottom = Number(settings.marginBottom);
			if(typeof settings.popOverViewWidth !== "undefined" && settings.popOverViewWidth) configs.popOverViewWidth = Number(settings.popOverViewWidth);
		}

		function initMapData(mapData) {
			if(typeof mapData === "undefined" || !mapData) throw "No mapData!";

			if(typeof mapData === "string") mapData = JSON.parse(mapData);
			mapData.linesArr.forEach(function(line) {
				line.nodesArr.forEach(function(node) {
					if(node.type !== "station") return;

					var existedStation = null;
					var lineNumber = line.lineId;
					node.lines = [lineNumber];

					for (var i = 0, l = allStationsArr.length; i < l; i++) {
						if (allStationsArr[i].nodeId === node.nodeId) {
							existedStation = allStationsArr[i];
							// console.log(existedStation);
							break;
						}
					}

					if (!existedStation) allStationsArr.push(new Station(node));
					else {
						existedStation.pushLine(lineNumber);
					}
					
				});
			});
		}

		function addTouchEvent() {
			var touchMetrics = null;
			var isDoubleTapEvent = false;
			var doubleTapTimeoutId;

			function captureTapEvent() {
				if(isDoubleTapEvent) {
					// double tap event...
					isDoubleTapEvent = false;
					clearTimeout(doubleTapTimeoutId);
				} else {
					isDoubleTapEvent = true;

					doubleTapTimeoutId = setTimeout(function() {
						isDoubleTapEvent = false;
						broadcast("onTouchStart");
					}, 400);
				}
			}

			function getEventClientPos(ev) {
				var x;
				var y;

				if(touch.isTouchDevice()) {
					x = ev.touches[0].pageX - configs.popOverViewWidth;
					y = ev.touches[0].pageY - configs.marginTop;
				}
				else {
					x = ev.pageX - configs.popOverViewWidth;
					y = ev.pageY - configs.marginTop;
				}

				return {
					x : (x < 0 ? -1 : x),
					y : (y < 0 ? -1 : y)
				};
			}

			mapView.addTouchMapEvent(function(ev) {
				touchMetrics = getEventClientPos(ev);
				// captureTapEvent();
				broadcast("onTouchStart");
			}, function(ev) {
				if(touchMetrics) touchMetrics = null;
			}, function(ev) {
				handleTouch();
			});

			function handleTouch() {
				var closestStation = getTheClosestStation(touchMetrics);
				var res = {};
				if(closestStation) {
					res = {
						x : closestStation.x,
						y : closestStation.y,
						id : configs.isLocalTesting ? closestStation.nameCn : closestStation.nodeId
					};
				}
				else {
					res = {
						x : touchMetrics.x,
						y : touchMetrics.y,
						id : -1
					};
				}

				broadcast("onTouch", res);
			}

			function getTheClosestStation(touchMetrics) {
				if(touchMetrics.x < 0 || touchMetrics.y < 0) return null;

				var closestStation = allStationsArr[0];
				var minDistance = Math.pow(closestStation.x - touchMetrics.x, 2) + Math.pow(closestStation.y - touchMetrics.y+20, 2);

				allStationsArr.forEach(function(station, index) {
					if(index === 0) return;

					var thisDistance = Math.pow(station.x - touchMetrics.x, 2) + Math.pow(station.y - touchMetrics.y+20, 2);
					if(minDistance > thisDistance) {
						closestStation = station;
						minDistance = thisDistance;
					}

				});

				return Math.sqrt(minDistance) > configs.maxGap ? null : closestStation;
			}
		}


		function Animation() {

			function init() {
				

				return {
					start : function(routeStationIdsArr,transferStationIdsArr) {
						transferStationIdsArr = JSON.stringify(transferStationIdsArr).replace(/\[|]/g,'').split(",");
						var routeStationsArr = [];

						function getStationById(id) {
							for(var i in allStationsArr) {
								var station = allStationsArr[i];
								if(station.nodeId == id) return station;
							}
							return null;
						}

						// setup routeStationsArr
						void function () {
							var lastLastLine = 0;
							var lastLines = [];
							var lastStationObj = null;

							function checkTransfer(obj) {
								var dist = "transfer";
								var station = obj.station;
								var isTransfer = true;
								var logStr = "log:";

								if (!lastStationObj) {
									lastLastLine = station.lines;
									
									logStr = logStr+"1station:"+JSON.stringify(station)+",lastLastLine:"+JSON.stringify(lastLastLine);
								} else {
									if (Object.prototype.toString.call(lastLastLine) === "[object Array]") {
										
										var sameLine = [];
										lastLastLine.forEach(function (lastLastLineNumber, index) {
											station.lines.forEach(function (lineNumber) {
												if (lineNumber === lastLastLineNumber) {
													sameLine.push(lineNumber);
												}
											});
										});
										if(sameLine.length > 0){
											lastLastLine = sameLine;
										}
										if(lastLastLine.length === 1){
											lastLastLine = lastLastLine[0];
										}
										transferStationIdsArr.forEach(function(transferStationId){
											//如果他是标记换乘站,那么它的所有线路都要带到下一步判断
											transferStationId = parseInt(transferStationId, 10);
											window.Android.clickImage("log:---6---"+"transferStationId:"+transferStationId);
											window.Android.clickImage("log:---7---"+"obj.station.nodeId:"+obj.station.nodeId);
											if(obj.station.nodeId === transferStationId){
												window.Android.clickImage("log:---5---换乘打上");
												obj.flag = dist;
												var sameLine = [];
												obj.station.lines.forEach(function (lineNumber) {
													sameLine.push(lineNumber);
												});
												if(sameLine.length === 1){
													lastLastLine = sameLine[0];
												} else if(sameLine.length > 1){
													lastLastLine = sameLine;
												}
											}
											
										});
									
										
									
										logStr = logStr+"\n,2Station:"+JSON.stringify(station)+",lastLastLine:"+JSON.stringify(lastLastLine);
									} else {
										station.lines.forEach(function (lineNumber) {
											if (lineNumber === lastLastLine) {
												isTransfer = false;
											}
										});
										
									
										if (isTransfer) {
											lastStationObj.flag = dist;
											var sameLine = [];
											lastLines.forEach(function (lastLineNumber) {
												station.lines.forEach(function (lineNumber) {
													if (lineNumber === lastLineNumber) {
														sameLine.push(lineNumber);
													}
												});
											});
											if(sameLine.length === 1){
												lastLastLine = sameLine[0];
											} else if(sameLine.length > 1){
												lastLastLine = sameLine;
											}
										}
										transferStationIdsArr.forEach(function(transferStationId){
											//如果他是标记换乘站,那么它的所有线路都要带到下一步判断
											transferStationId = parseInt(transferStationId, 10);
											window.Android.clickImage("log:---6---"+"transferStationId:"+transferStationId);
											window.Android.clickImage("log:---7---"+"obj.station.nodeId:"+obj.station.nodeId);
											if(obj.station.nodeId === transferStationId){
												window.Android.clickImage("log:---5---换乘打上");
												obj.flag = dist;
												var sameLine = [];
												obj.station.lines.forEach(function (lineNumber) {
													sameLine.push(lineNumber);
												});
												if(sameLine.length === 1){
													lastLastLine = sameLine[0];
												} else if(sameLine.length > 1){
													lastLastLine = sameLine;
												}
											}
											
										});
										logStr = logStr+"3station:"+JSON.stringify(station)+",lastLastLine:"+JSON.stringify(lastLastLine);
									}
								}
								window.Android.clickImage(logStr+",isTransfer:"+isTransfer);
								window.Android.clickImage("log:---4---"+JSON.stringify(obj));
								lastLines = station.lines;
								lastStationObj = obj;
							}

							if(typeof routeStationIdsArr === "string") routeStationIdsArr = routeStationIdsArr.replace(/\s/g, "").split(",");

							routeStationIdsArr.forEach(function (id, index) {
								var station = getStationById(parseInt(id, 10));

								if (!station) return;

								var result = {};
								if (!index) result.flag = "from";
								else if (index === routeStationIdsArr.length - 1) result.flag = "to";
								result.station = station;
								checkTransfer(result);

								if (!index) result.flag = "from";
								else if (index === routeStationIdsArr.length - 1) result.flag = "to";

								routeStationsArr.push(result);
							});
						}();

						// if(!mapView.hasOverlay()) mapView.createOverlay();
						// mapView.showOverlay(function() {

							if(!mapView.hasPinsWrapper()) mapView.createPinsWrapper();

							// get highlighted metric
							void function() {
								var metrics = function() {
									var metrics = {};
									var top;
									var right;
									var bottom;
									var left;

									routeStationsArr.forEach(function(obj, index) {
										var station = obj.station;
										if(station) {
											if(index === 0) {
												left = right = station.x;
												top = bottom = station.y;
												return;
											}
											if(right < station.x) right = station.x;
											if(left > station.x) left = station.x;
											if(top > station.y) top = station.y;
											if(bottom < station.y) bottom = station.y;
										}
									});

									metrics.x = left + configs.popOverViewWidth;
									metrics.y = top + configs.marginTop;
									metrics.w = right - left;
									metrics.h = bottom - top;

									if(metrics.w < 10) metrics.w = 10;
									if(metrics.h < 10) metrics.h = 10;

									return metrics;
								}();

								// get scaleRatio, translateX, translateY
								void function() {
									var translateX;
									var translateY;
									var visibleWidth = configs.deviceWidth;
									var visibleHeight = configs.deviceHeight - configs.marginTop - configs.marginBottom;
									var scaleRatio = Math.min(visibleWidth / metrics.w, visibleHeight / metrics.h);

									if(scaleRatio > 1) scaleRatio = 1;
									else scaleRatio = Number(scaleRatio.toFixed(2));

									translateX = parseInt(metrics.x - ((visibleWidth - (metrics.w * scaleRatio)) * .5));
									translateY = parseInt((metrics.y - ((visibleHeight - (metrics.h * scaleRatio)) * .5)) - (configs.marginTop / scaleRatio));

									if(translateX < 0) translateX = 0;
									if(translateY < 0) translateY = 0;

									broadcast("onAnimationStart", {
										scaleRatio : scaleRatio,
										translateX : translateX,
										translateY : translateY
									});

								}();
							}();

							// start animation
							void function() {
								var index = 0;
								var length = routeStationsArr.length;

								// prevent touch event
								mapView.addTouchDocumentEvent(function(ev) {
									ev.preventDefault();
								}, function(ev) {
									ev.preventDefault();
								}, function(ev) {
									ev.preventDefault();
								});

								void function next() {
									if(index === length) {
										// end
										animationId = null;
										mapView.removeTouchDocumentEvent();
										return;
									}

									var stationObj = routeStationsArr[index];
									mapView.addPin(stationObj.station, stationObj.flag);
									
									index++;
									animationId = setTimeout(next, 100);
								}();
							}();

						// });
					},

					stop : function() {
						if(animationId) clearTimeout(animationId);
					}
				}
			}

			return {
				getInstance : init
			};
		}

		function startAnimation(stationIdsArr,stationIdsArr2) {
			var animation = Animation().getInstance();
			clearHighlighting();
			animation.stop();
			animation.start(stationIdsArr,stationIdsArr2);
		}

		function clearHighlighting() {
			// if(mapView.hasOverlay()) mapView.hideOverlay();
			if(mapView.hasPinsWrapper()) {
				pinsCollection.clear();
				mapView.removePinsWrapper();
			}
		}

		function addInterface() {
			window.WEBVIEW.show = mapView.showWebview;
			window.WEBVIEW.startAnimation = startAnimation;
			window.WEBVIEW.clearHighlighting = clearHighlighting;
			window.WEBVIEW.scale = pinsCollection.scaleAllPins;
			window.WEBVIEW.pick = mapView.pick;
			window.WEBVIEW.select = mapView.select;
			window.WEBVIEW.deselect = mapView.deselect;
		}
	}

	init();

}();

